function [imSkeletonAlign minutiaArrayAlign im2Align offset ang jadra] = align_manually(imDatabase, imVerify, inputSkel, inputMinutia, im2)
%Funkce slouzi k rucnimu zarovnani otisku na sebe. Vstupem jsou dva obrazky
%otisku, ktere slouzi pouze pro zadani vyznamnych bodu 
%(muze to byt primo obrazek otisku nebo otisky po Gaborove filtraci apod.).
%Dale je vstupem kostra verifikovaneho obrazku, pole s markanty
%verifikovaneho otisku a obrazek samotneho otisku.
%
%Pri zarovnavani jsou ukazany oba otisky. Nejprve je v nich treba zadat
%stejny bod, podle ktereho bude verifikovany otisk otacen (jadro). Po zadani  
%jadra je postupne v levem a pak v pravem otisku zadat 2 markanty, ktere si
%odpovidaji.
%
%[imSkeletonAlign minutiaArrayAlign im2Align offset ang jadra] =
%       align_manually(imDatabase, imVerify, inputSkel, inputMinutia, im2)
%
%       VSTUPY: imDAtabase - obrazek databazoveho otisku
%               imVerify - obrazek verifikovaneho otisku
%               inputSkel - kostra verifikovaneho otisku
%               inputMinutia - pole s markanty ver. otisku
%               im2 - obrazek verifikovaneho otisku
%
%       VYSTUPY: imSkeletonAlign - kostra ver. otisku zarovnana s
%                   databazovym otiskem
%                minutiaArrayAlign - pole s markanty, zarovnane
%                   s databazovym otiskem
%                im2Align - obrazek zarovnaneho ver. otisku
%                offset - posun mezi obrazky
%                ang - uhel natoceni
%                jadra - polohy jader v obrazcich (nejdrive v databazovem, potom verifikovanem)

f = figure; p1 = subplot(121);
imshow(imDatabase); hold on
p2 = subplot(122);
imshow(imVerify); hold on

suptitle('Zadejte jadro otisku.')
jadra = ginput(2);

figure(f);
axes(p1)
plot(jadra(1, 1), jadra(1, 2), 'r.', 'markersize', 20);
axes(p2)
plot(jadra(2, 1), jadra(2, 2), 'r.', 'markersize', 20);

suptitle('Zadejte dva markanty v levem obraze.')
ii = 0;
while true
    ii = ii+1;
    markanty1(ii, :) = ginput(1);
    axes(p1)
    plot(markanty1(ii, 1), markanty1(ii, 2), '.g', 'markersize', 15);
    if ii == 2, break; end
end

suptitle('Zadejte dva markanty v pravem obraze.')
ii = 0;
while true
    ii = ii+1;
    markanty2(ii, :) = ginput(1);
    axes(p2)
    plot(markanty2(ii, 1), markanty2(ii, 2), '.g', 'markersize', 15);
    if ii == 2, break; end
end

a1 = markanty1(1, :) - jadra(1, :);
a1 = a1(1) + a1(2)*1i;

b1 = markanty1(2, :) - jadra(1, :);
b1 = b1(1) + b1(2)*1i;

a2 = markanty2(1, :) - jadra(2, :);
a2 = a2(1) + a2(2)*1i;

b2 = markanty2(2, :) - jadra(2, :);
b2 = b2(1) + b2(2)*1i;

u1 = normAngle(angle(a2) - angle(a1));
u2 = normAngle(angle(b2) - angle(b1));
% uhly = [u1 u2];

ang = mean([u1, u2])/pi*180;
offset = jadra(1, :) - jadra(2, :);


%reconstruction good fp skeleton
inputSkelreconstruct = imcomplement(full(inputSkel));
inputSkelreconstruct = imshift(inputSkelreconstruct, [offset(1); offset(2)]);
inputSkelreconstruct = imrotatearound(inputSkelreconstruct, ang, jadra(1, :)');
inputSkelreconstruct = imcomplement(full(inputSkelreconstruct));
imSkeletonAlign=inputSkelreconstruct;

%reconstruct good minutiae
inputMinRec = imshift(inputMinutia, [offset(1); offset(2)]);
inputMinRec = imrotatearound(inputMinRec, ang, jadra(1, :)');
minutiaArrayAlign=inputMinRec;

% [mA1y mA1x] = find(inputMinRec == 1);
% [mA2y mA2x] = find(inputMinRec == 2);
% minAlign = [mA1x mA1y ones(length(mA1x), 1); mA2x mA2y ones(length(mA2x), 1)*2];

% zarovnani vstupniho obrazku s databazovym
im2Align = imrotatearound(imshift(im2, offset'), ang, jadra(1,:)');

function ang = normAngle(ang)
if ang < 0
    ang = 2*pi + ang;
end
